@@ -1,8 +1,8 @@
 # $File: //member/autrijus/PAR/lib/PAR.pm $ $Author: autrijus $
-# $Revision: #56 $ $Change: 9630 $ $DateTime: 2004/01/08 10:59:30 $ vim: expandtab shiftwidth=4
+# $Revision: #68 $ $Change: 10399 $ $DateTime: 2004/03/16 16:25:41 $ vim: expandtab shiftwidth=4
 
 package PAR;
-$PAR::VERSION = '0.79';
+$PAR::VERSION = '0.80';
 
 use 5.006;
 use strict;
@@ -15,7 +15,7 @@ PAR - Perl Archive Toolkit
 
 =head1 VERSION
 
-This document describes version 0.79 of PAR, released January 8, 2004.
+This document describes version 0.80 of PAR, released March 17, 2004.
 
 =head1 SYNOPSIS
 
@@ -119,7 +119,7 @@ Please see L</SYNOPSIS> for most typical use cases.
 Settings in F<META.yml> packed inside the PAR file may affect PAR's
 operation.  For example, F<pp> provides the C<-C> (C<--clean>) option
 to control the default behavior of temporary file creation.
- 
+
 Currently, F<pp>-generated PAR files may attach four PAR-specific
 attributes in F<META.yml>:
 
@@ -152,15 +152,18 @@ If both are not set, use C<I<TEMP>\par-I<USER>\temp-I<MTIME>\>
 as the I<PAR_TEMP>, reusing any existing files inside.  I<MTIME>
 is the last-modified timestamp of the program.
 
+=back
+
 =cut
 
 use vars qw(@PAR_INC);  # explicitly stated PAR library files
 use vars qw(%PAR_INC);  # sets {$par}{$file} for require'd modules
 use vars qw(@LibCache %LibCache);       # I really miss pseudohash.
-use vars qw($LastAccessedPAR);
+use vars qw($LastAccessedPAR $LastTempFile);
 
 my $ver  = $Config{version};
 my $arch = $Config{archname};
+my $is_insensitive_fs = (-s $0 and (-s lc($0) || -1) == (-s uc($0) || -1));
 my ($par_temp, $progname);
 
 sub import {
@@ -193,17 +196,22 @@ sub import {
         # XXX - handle META.yml here!
         push @PAR_INC, unpar($0, undef, undef, 1);
 
+        _extract_inc($0) unless $ENV{PAR_CLEAN};
+
         my $zip = $LibCache{$0};
-        my $member = $zip->memberNamed("script/main.pl")
-                  || $zip->memberNamed("main.pl");
+        my $member = _first_member( $zip,
+            "script/main.pl",
+            "main.pl",
+        );
 
         # finally take $ARGV[0] as the hint for file to run
         if (defined $ARGV[0] and !$member) {
-            $member  = $zip->memberNamed("script/$ARGV[0]")
-                    || $zip->memberNamed("script/$ARGV[0].pl")
-                    || $zip->memberNamed("$ARGV[0]")
-                    || $zip->memberNamed("$ARGV[0].pl")
-                or die qq(Can't open perl script "$ARGV[0]": No such file or directory);
+            $member = _first_member( $zip,
+                "script/$ARGV[0]",
+                "script/$ARGV[0].pl",
+                $ARGV[0],
+                "$ARGV[0].pl",
+            ) or die qq(Can't open perl script "$ARGV[0]": No such file or directory);
             shift @ARGV;
         }
         elsif (!$member) {
@@ -214,6 +222,17 @@ sub import {
     }
 }
 
+sub _first_member {
+    my $zip = shift;
+    my %names = map { ( $_->fileName => $_ ) } $zip->members;
+    my %lc_names = map { ( lc($_->fileName) => $_ ) } $zip->members;
+    foreach my $name (@_) {
+        return $names{$name} if $names{$name};
+        return $lc_names{lc($name)} if $is_insensitive_fs and $lc_names{lc($name)};
+    }
+    return;
+}
+
 sub _run_member {
     my $member = shift;
     my $clear_stack = shift;
@@ -235,6 +254,20 @@ sub _run_member {
     { do 'main'; die $@ if $@; exit }
 }
 
+sub _extract_inc {
+    my $file = shift;
+    my $inc = "$par_temp/inc";
+
+    if (!-d $inc) {
+        for (1 .. 10) { mkdir("$inc.lock", 0755) and last; sleep 1 }
+        Archive::Zip->new($file)->extractTree( '' => "$inc/" );
+        rmdir("$inc.lock");
+    }
+
+    unshift @INC, grep -d, map join('/', $inc, @$_),
+        [ 'lib' ], [ 'arch' ], [ $arch ], [ $ver ], [ $ver, $arch ], [];
+}
+
 sub find_par {
     my ($self, $file, $member_only) = @_;
 
@@ -250,6 +283,7 @@ sub find_par {
         }
         my $rv = unpar($path, $file, $member_only, 1) or next;
         $PAR_INC{$path}{$file} = 1;
+        $INC{$file} = $LastTempFile if (lc($file) =~ /^(?!tk).*\.pm$/);
         return $rv;
     }
 
@@ -274,7 +308,7 @@ sub read_file {
     my $file = pop;
 
     foreach my $zip (@LibCache) {
-        my $member = $zip->memberNamed($file) or next;
+        my $member = _first_member($zip, $file) or next;
         return scalar $member->contents;
     }
 
@@ -346,7 +380,7 @@ sub unpar {
         push @LibCache, $zip;
         $LibCache{$_[0]} = $zip;
 
-        foreach my $member ( $zip->members(
+        foreach my $member ( $zip->membersMatching(
             "^par/(?:$Config{version}/)?(?:$Config{archname}/)?"
         ) ) {
             next if $member->isDirectory;
@@ -360,16 +394,19 @@ sub unpar {
 
     return @rv unless defined $file;
 
-    my $member = $zip->memberNamed("lib/$file")
-              || $zip->memberNamed("arch/$file")
-              || $zip->memberNamed("$arch/$file")
-              || $zip->memberNamed("$ver/$file")
-              || $zip->memberNamed("$ver/$arch/$file")
-              || $zip->memberNamed($file) or return;
+    my $member = _first_member($zip,
+        "lib/$file",
+        "arch/$file",
+        "$arch/$file",
+        "$ver/$file",
+        "$ver/$arch/$file",
+        $file,
+    ) or return;
 
     return $member if $member_only;
 
-    my ($fh, $is_new) = _tempfile($member->crc32String . ".pm");
+    my ($fh, $is_new);
+    ($fh, $is_new, $LastTempFile) = _tempfile($member->crc32String . ".pm");
     die "Bad Things Happened..." unless $fh;
 
     if ($is_new) {
@@ -401,7 +438,19 @@ sub _set_par_temp {
         my $stmpdir = File::Spec->catdir($path, "par-$username");
         mkdir $stmpdir, 0755;
         if (!$ENV{PAR_CLEAN} and my $mtime = (stat($progname))[9]) {
-            $stmpdir = File::Spec->catdir($stmpdir, "cache-$mtime");
+            my $ctx = eval { require Digest::SHA1; Digest::SHA1->new }
+                   || eval { require Digest::MD5; Digest::MD5->new };
+                
+            if ($ctx and open(my $fh, "<$progname")) {
+                binmode($fh);
+                $ctx->addfile($fh);
+                close($fh);
+            }
+
+            $stmpdir = File::Spec->catdir(
+                $stmpdir,
+                "cache-" . ( $ctx ? $ctx->hexdigest : $mtime )
+            );
         }
         else {
             $ENV{PAR_CLEAN} = 1;
@@ -420,15 +469,17 @@ sub _tempfile {
     if ($ENV{PAR_CLEAN} or !@_) {
         require File::Temp;
 
-        # under Win32, the file is created with O_TEMPORARY,
-        # and will be deleted by the C runtime; having File::Temp
-        # delete it has the only effect of giving ugly warnings
-        my $fh = File::Temp::tempfile(
-            DIR     => $par_temp,
-            UNLINK  => ($^O ne 'MSWin32'),
-        ) or die "Cannot create temporary file: $!";
-        binmode($fh);
-        return ($fh, 1);
+        if (defined &File::Temp::tempfile) {
+            # under Win32, the file is created with O_TEMPORARY,
+            # and will be deleted by the C runtime; having File::Temp
+            # delete it has the only effect of giving ugly warnings
+            my ($fh, $filename) = File::Temp::tempfile(
+                DIR     => $par_temp,
+                UNLINK  => ($^O ne 'MSWin32'),
+            ) or die "Cannot create temporary file: $!";
+            binmode($fh);
+            return ($fh, 1, $filename);
+        }
     }
 
     require File::Spec;
@@ -436,12 +487,12 @@ sub _tempfile {
     if (-r $filename) {
         open my $fh, '<', $filename or die $!;
         binmode($fh);
-        return ($fh, 0);
+        return ($fh, 0, $filename);
     }
 
     open my $fh, '+>', $filename or die $!;
     binmode($fh);
-    return ($fh, 1);
+    return ($fh, 1, $filename);
 }
 
 sub _set_progname {
