@@ -1,6 +1,6 @@
 #!/usr/local/bin/perl
 # $File: //member/autrijus/PAR/script/par.pl $ $Author: autrijus $
-# $Revision: #94 $ $Change: 9608 $ $DateTime: 2004/01/04 20:17:28 $ vim: expandtab shiftwidth=4
+# $Revision: #101 $ $Change: 10328 $ $DateTime: 2004/03/05 06:51:23 $ vim: expandtab shiftwidth=4
 
 package __par_pl;
 
@@ -32,18 +32,18 @@ To manipulate a I<PAR distribution>:
 
 To use F<Hello.pm> from F<./foo.par>:
 
-    % par.pl -A./foo.par -MHello 
+    % par.pl -A./foo.par -MHello
     % par.pl -A./foo -MHello    # the .par part is optional
 
 Same thing, but search F<foo.par> in the F<@INC>;
 
-    % par.pl -Ifoo.par -MHello 
+    % par.pl -Ifoo.par -MHello
     % par.pl -Ifoo -MHello      # ditto
 
 Run F<test.pl> or F<script/test.pl> from F<foo.par>:
 
     % par.pl foo.par test.pl    # looks for 'main.pl' by default,
-                                # otherwise run 'test.pl' 
+                                # otherwise run 'test.pl'
 
 To make a self-containing script containing a PAR file :
 
@@ -97,7 +97,7 @@ PAR file:
     % ./myapp                   # run it anywhere without perl binaries
 
 With the C<--par-options> flag, generated binaries can act as C<parl>
-to pack new binaries: 
+to pack new binaries:
 
     % ./myapp --par-options -Omyap2 myapp.par   # identical to ./myapp
     % ./myapp --par-options -Omyap3 myap3.par   # now with different PAR
@@ -151,10 +151,10 @@ followed by a 8-bytes magic string: "C<\012PAR.pm\012>".
 =cut
 
 my ($par_temp, $progname, @tmpfile);
-END { if (@tmpfile) {
+END { if ($ENV{PAR_CLEAN}) {
     unlink @tmpfile;
     rmdir $par_temp;
-    $par_temp =~ s{[^\\/]*$}{};
+    $par_temp =~ s{[^\\/]*[\\/]?$}{};
     rmdir $par_temp;
 } }
 
@@ -163,10 +163,17 @@ BEGIN {
 
     eval {
 
-if ($ENV{PAR_ARGV_0}) {
+_par_init_env();
+
+if (exists $ENV{PAR_ARGV_0} and $ENV{PAR_ARGV_0} ) {
     @ARGV = map $ENV{"PAR_ARGV_$_"}, (1 .. $ENV{PAR_ARGC} - 1);
     $0 = $ENV{PAR_ARGV_0};
 }
+else {
+    for (keys %ENV) {
+        delete $ENV{$_} if /^PAR_ARGV_/;
+    }
+}
 
 my $quiet = !$ENV{PAR_DEBUG};
 
@@ -177,10 +184,14 @@ my %Config = (
     _delim      => ($^O =~ /^MSWin|OS2/ ? '\\' : '/'),
 );
 
-_par_init_env();
 _set_progname();
 _set_par_temp();
 
+# Append ".exe" to $0 if it was missing
+$0 .= $Config{_exe} if $Config{_exe}
+    and $progname =~ /\Q$Config{_exe}\E$/
+    and $0 !~ /\Q$Config{_exe}\E$/;
+
 # Magic string checking and extracting bundled modules {{{
 my ($start_pos, $data_pos);
 {
@@ -242,6 +253,7 @@ my ($start_pos, $data_pos);
             $PAR::Heavy::ModuleCache{$fullname} = {
                 buf => $buf,
                 crc => $crc,
+                name => $fullname,
             };
         }
         read _FH, $buf, 4;
@@ -314,7 +326,7 @@ my ($start_pos, $data_pos);
 
 # Argument processing {{{
 my @par_args;
-my ($out, $bundle);
+my ($out, $bundle, $logfh);
 
 $quiet = 0 unless $ENV{PAR_DEBUG};
 
@@ -328,7 +340,7 @@ if (!$start_pos or ($ARGV[0] eq '--par-options' && shift)) {
         v   verify_par
     );
     while (@ARGV) {
-        $ARGV[0] =~ /^-([AIMOBbqpiusv])(.*)/ or last;
+        $ARGV[0] =~ /^-([AIMOBLbqpiusv])(.*)/ or last;
 
         if ($1 eq 'I') {
             unshift @INC, $2;
@@ -351,6 +363,9 @@ if (!$start_pos or ($ARGV[0] eq '--par-options' && shift)) {
         elsif ($1 eq 'q') {
             $quiet = 1;
         }
+        elsif ($1 eq 'L') {
+            open $logfh, ">>", $2 or die "XXX: Cannot open log: $!";
+        }
 
         shift(@ARGV);
 
@@ -432,7 +447,7 @@ if ($out) {
         } grep {
             !/BSDPAN/
         } grep {
-            ($bundle ne 'site') or 
+            ($bundle ne 'site') or
             ($_ ne $Config::Config{archlibexp} and
              $_ ne $Config::Config{privlibexp});
         } @INC;
@@ -472,7 +487,10 @@ if ($out) {
 
             next unless defined $name and not $written{$name}++;
             next if !ref($file) and $file =~ /\.\Q$lib_ext\E$/;
-            outs(qq(Packing "$file"...));
+            outs( join "",
+                qq(Packing "), ref $file ? $file->{name} : $file,
+                qq("...)
+            );
 
             my $content;
             if (ref($file)) {
@@ -608,7 +626,17 @@ sub _set_par_temp {
         my $stmpdir = "$path$Config{_delim}par-$username";
         mkdir $stmpdir, 0755;
         if (!$ENV{PAR_CLEAN} and my $mtime = (stat($progname))[9]) {
-            $stmpdir .= "$Config{_delim}cache-$mtime";
+            my $ctx = eval { require Digest::SHA1; Digest::SHA1->new }
+                   || eval { require Digest::MD5; Digest::MD5->new };
+
+            if ($ctx and open(my $fh, "<$progname")) {
+                binmode($fh);
+                $ctx->addfile($fh);
+                close($fh);
+            }
+
+            $stmpdir .= "$Config{_delim}cache-"
+                     . ( $ctx ? $ctx->hexdigest : $mtime );
         }
         else {
             $ENV{PAR_CLEAN} = 1;
@@ -626,7 +654,7 @@ sub _set_par_temp {
 sub _tempfile {
     my ($ext, $crc) = @_;
     my ($fh, $filename);
-    
+
     $filename = "$par_temp/$crc$ext";
 
     if ($ENV{PAR_CLEAN}) {
@@ -650,6 +678,7 @@ sub _set_progname {
 
     if (-s "$progname$Config{_exe}") {
         $ENV{PAR_PROGNAME} = $progname = "$progname$Config{_exe}";
+        $0 .= $Config{_exe};
         return $progname;
     }
 
@@ -665,20 +694,39 @@ sub _set_progname {
 }
 
 sub _par_init_env {
-    return if $ENV{PAR_INITIALIZED}++;
+    if ( $ENV{PAR_INITIALIZED}++ == 1 ) {
+        return;
+    } else {
+        $ENV{PAR_INITIALIZED} = 2;
+    }
+
+    for (qw( SPAWNED TEMP CLEAN DEBUG CACHE PROGNAME ARGC ARGV_0 ) ) {
+        delete $ENV{'PAR_'.$_};
+    }
+    for (qw/ TEMP CLEAN DEBUG /) {
+        $ENV{'PAR_'.$_} = $ENV{'PAR_GLOBAL_'.$_} if exists $ENV{'PAR_GLOBAL_'.$_};
+    }
 
     my $par_clean = "__ENV_PAR_CLEAN__               ";
 
     if ($ENV{PAR_TEMP}) {
         delete $ENV{PAR_CLEAN};
     }
-    elsif (!$ENV{PAR_CLEAN}) {
+    elsif (!exists $ENV{PAR_GLOBAL_CLEAN}) {
         my $value = substr($par_clean, 12 + length("CLEAN"));
         $ENV{PAR_CLEAN} = $1 if $value =~ /^PAR_CLEAN=(\S+)/;
     }
 }
 
-sub outs { warn("@_\n") unless $quiet }
+sub outs {
+    return if $quiet;
+    if ($logfh) {
+        print $logfh "@_\n";
+    }
+    else {
+        warn "@_\n";
+    }
+}
 
 sub init_inc {
     require Config;
